//--------------------------------------------------------------------
// Pivot Points - Daily (Shifted).mq4
// Copyright  2009, SwingTree Ltd
// 
// This indicator plots horizontal lines at daily pivot levels R3, R2,
// R1, Pivot, S1, S2 & S3, using a day that is advanced (or retarded)
// from the server time (called Pivot Day).
// 
// The only input is the integer "ShiftHrs", which gives the number of
// hours to advance the pivot day. A negative value delays the day for
// pivot calculation. For example, if your server is on New York time
// (GMT-5) and you want pivot points in London time (GMT), then
// ShiftHrs is set to +5 (the default in the code).
//
// Saturday and Sunday trading is not ignored (after all it was real
// trading). If the pivot day shift results in data appearing on
// Saturday, it was really Friday where it occured, so the indicator
// treats it as part of Friday. If the pivot day shift results in data
// appearing on Sunday, it was really Monday where it occured, so the
// indicator treats it as Monday.
//--------------------------------------------------------------------
 
#property copyright "Copyright  2009, SwingTree Ltd"
 
#define R3_NAME "R3"
#define R2_NAME "R2"
#define R1_NAME "R1"
#define PIVOT_NAME "PP"
#define S1_NAME "S1"
#define S2_NAME "S2"
#define S3_NAME "S3"
 
#define FONT "Arial"
 
#define R3_COL Red
#define R2_COL Red
#define R1_COL Red
#define PIVOT_COL DimGray
#define S1_COL Green
#define S2_COL Green
#define S3_COL Green
 
#property indicator_chart_window
#property indicator_buffers 7
#property indicator_color1 R3_COL
#property indicator_color2 R2_COL
#property indicator_color3 R1_COL
#property indicator_color4 PIVOT_COL
#property indicator_color5 S1_COL
#property indicator_color6 S2_COL
#property indicator_color7 S3_COL

// Input(s)
extern int ShiftHrs = 2;   // Pivot day shift
                           // positive value moves pivot day earlier
 
extern bool ShowLevel3 = false;
extern bool DrawPPTransition = false;
extern bool ShowComment = true;
// Buffers for levels
double Res3[], Res2[], Res1[], Pivot[], Sup1[], Sup2[], Sup3[];
 
double PDayHigh, PDayLow, PDayClose;
string ThisSymbol;
datetime BarTime, PivotDayStartTime;
int VisibleBars, DayStartBar, LeftMostBar, RightMostBar;
 
 
//--------------------------------------------------------------------
// Initialization
//--------------------------------------------------------------------
int init()
{
   // Attach indicator arrays to buffers
   SetIndexBuffer( 0, Res3);
   SetIndexBuffer( 1, Res2);
   SetIndexBuffer( 2, Res1);
   SetIndexBuffer( 3, Pivot);
   SetIndexBuffer( 4, Sup1);
   SetIndexBuffer( 5, Sup2);
   SetIndexBuffer( 6, Sup3);
 
   // Set styles
   SetIndexStyle( 0, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexStyle( 1, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexStyle( 2, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexStyle( 3, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexStyle( 4, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexStyle( 5, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexStyle( 6, DRAW_LINE, STYLE_SOLID, 1);
 
   // Set empty values
   SetIndexEmptyValue( 0, EMPTY_VALUE );
   SetIndexEmptyValue( 1, EMPTY_VALUE );
   SetIndexEmptyValue( 2, EMPTY_VALUE );
   SetIndexEmptyValue( 3, EMPTY_VALUE );
   SetIndexEmptyValue( 4, EMPTY_VALUE );
   SetIndexEmptyValue( 5, EMPTY_VALUE );
   SetIndexEmptyValue( 6, EMPTY_VALUE );
 
   // Catch bad input
   if( MathAbs( ShiftHrs ) > 23 )
   {
      Alert( "ShiftHrs is too large. Reset to 0 hrs. ");
      ShiftHrs = 0;
   }

   ThisSymbol = Symbol();
   PivotDayStartTime = Time[Bars-1];         // the first bar
   
   return(0);
}
 
//--------------------------------------------------------------------
//| De-initialization                                                |
//--------------------------------------------------------------------
int deinit()
{
   // Remove texts
   ObjectDelete( R3_NAME );
   ObjectDelete( R2_NAME );
   ObjectDelete( R1_NAME );
   ObjectDelete( PIVOT_NAME );
   ObjectDelete( S1_NAME );
   ObjectDelete( S2_NAME );
   ObjectDelete( S3_NAME );
   Comment(""); 
   return(0);
}
 
//--------------------------------------------------------------------
//| Main iteration                                                   |
//--------------------------------------------------------------------
int start()
{
   int i, Count;
   double Range;
 
   i = Bars - IndicatorCounted() - 1;
   if(Period()>60) return(0);
   while(i >= 0)
   {
      // If the pivot day changes...
      if( PivotDay( Time[i+1], ShiftHrs ) != PivotDay( Time[i], ShiftHrs ) )
      {
         // Determine High & Low for the previous Pivot Day
         Count = iBarShift( NULL, 0, PivotDayStartTime ) - i;           // number of bars in the day
         PDayClose = Close[i+1];
         PDayHigh = High[ iHighest( NULL, 0, MODE_HIGH, Count, i+1 ) ]; // Pivot Day high
         PDayLow = Low[ iLowest( NULL, 0, MODE_LOW, Count, i+1 ) ];     // Pivot Day low
 
         // Pivot calculations
         Pivot[i] = ( PDayHigh + PDayLow + Close[i+1] ) / 3;    // Pivot point
         Range = PDayHigh - PDayLow;
         
         Res1[i] = 2 * Pivot[i] - PDayLow;                     // R1
         Res2[i] = Pivot[i] + Range;                           // R2
         Sup1[i] = 2 * Pivot[i] - PDayHigh;                    // S1
         Sup2[i] = Pivot[i] - Range;                           // S2
         
         if(ShowLevel3==true) {
            Sup3[i] = Sup1[i] - Range;                         // S3
            Res3[i] = Res1[i] + Range;                         // R3    
            Res3[i+1] = EMPTY_VALUE;
            Sup3[i+1] = EMPTY_VALUE;
         }
         
         // Don't draw the transition between levels
         Res2[i+1] = EMPTY_VALUE;
         Res1[i+1] = EMPTY_VALUE;
         Pivot[i+1] = EMPTY_VALUE;
         Sup1[i+1] = EMPTY_VALUE;
         Sup2[i+1] = EMPTY_VALUE;

         
      // Remember when the Day changed over
         PivotDayStartTime = Time[i];
      }
      else     // no change to pivot levels
      {
         Res3[i] = Res3[i+1];
         Res2[i] = Res2[i+1];
         Res1[i] = Res1[i+1];
         Pivot[i] = Pivot[i+1];
         Sup1[i] = Sup1[i+1];
         Sup2[i] = Sup2[i+1];
         Sup3[i] = Sup3[i+1];
      }

      VisibleBars = WindowBarsPerChart();
      BarTime = Time[i];
      i--;
   }

   //------ Comment for upper left corner
   if (ShowComment) {
      string comment= ""; 
   
      comment= comment + "Range: Yesterday "+DoubleToStr(MathRound(Point),0)   +" pips, Today "+DoubleToStr(MathRound(Point),0)+" pips" + "\n";
      comment= comment + "Highs: Yesterday "+DoubleToStr(PDayHigh,Digits)  +", Today "+DoubleToStr(High[ iHighest( NULL, 0, MODE_HIGH, 10, 0 ) ],Digits) +"\n";
      comment= comment + "Lows:  Yesterday "+DoubleToStr(PDayLow,Digits)   +", Today "+DoubleToStr("a",Digits)  +"\n";
      comment= comment + "Close: Yesterday "+DoubleToStr(PDayClose,Digits) + "\n";
      comment= comment + "Pivot: " + DoubleToStr(Pivot[1],Digits) + "\n";
      comment= comment + "Supports: " + DoubleToStr(Sup1[1],Digits) + "/" + DoubleToStr(Sup2[1],Digits) + "\n";
      comment= comment + "Resistance: " + DoubleToStr(Res1[1],Digits) + "/" + DoubleToStr(Res2[1],Digits);
   // comment= comment + "Fibos: " + DoubleToStr(yesterday_low + q*0.382, Digits) + ", " + DoubleToStr(yesterday_high - q*0.382,Digits) + "\n";
      
      Comment(comment); 
   }
   return(0);
}
 
 
//--------------------------------------------------------------------
// int PivotDay( datetime BarTime, datetime ShiftHrs )
// Returns the day of the week for pivot point calculations.
// datetime BarTime: time stamp of the bar of interest
// datetime Shift:   the pivot time - server time shift
//                   i.e. if the time for pivot calculation is ahead
//                   of server time, the shift is positive.
//--------------------------------------------------------------------
int PivotDay( datetime BarTime, datetime ShiftHrs )
{
   int PDay = TimeDayOfWeek( BarTime + ShiftHrs * 3600 );
 
      if( PDay == 0 ) PDay = 1;      // Count Sunday as Monday
      //if( PDay == 6 ) PDay = 5;      // Count Saturday as Friday
 
   return( PDay );
}